/*
 * Copyright (c) 2005 Your Corporation. All Rights Reserved.
 */
package math.complex;

/**
 * User: lyon
 * Date: Nov 13, 2005
 * Time: 8:00:08 AM
 * Copyright DocJava, Inc. 2005.
 */
public class ComplexSinglePrecisionArray {
    private float re[];
    private float im[];

    public ComplexSinglePrecisionArray() {
    }

    public ComplexSinglePrecisionArray(int n) {
        re = new float[n];
        im = new float[n];
    }

    // a = a + b, a=a.plus(b)
    public void plus(ComplexSinglePrecisionArray b) {
        for (int i = 0; i < re.length; i++) {
            re[i] += b.getRe(i);
        }
        for (int i = 0; i < re.length; i++) {
            im[i] += b.getIm(i);
        }
    }

    //todo must test the operations! All are untested!
    public ComplexSinglePrecisionArray getCopy() {
        int n = re.length;
        ComplexSinglePrecisionArray c = new ComplexSinglePrecisionArray(n);
        System.arraycopy(re, 0, c.getRe(), 0, n);
        System.arraycopy(im, 0, c.getIm(), 0, n);
        return c;
    }

    /**
     * Mask the image kernal with the real and imaginary numbers.
     * Assume the image kernal ranges from 0..255. Make it a unitary
     * filter by dividing by 255.
     *
     * @param r
     */
    public void scaleAndMask(short r[][]) {
        int w = r.length;
        int h = r[0].length;
        for (int x = 0; x < w; x++) {
            for (int y = 0; y < h; y++) {
                int i = x + y * w;
                re[i] *= r[x][y] / 255f;
                im[i] *= r[x][y] / 255f;
            }
        }
    }

    public void scale(float sc) {
        for (int i = 0; i < re.length; i++)
            re[i] *= sc;
        for (int i = 0; i < re.length; i++)
            im[i] *= sc;
    }

    //todo test the below code...
    // a = a * b, a=a.times(b);
    //(x1 + iy1)(x2 + iy2) =
    //      x1        *  x2    - y1        * y2 +
    //    i(y1        *  x2    + x1        * y2)
    public void times(ComplexSinglePrecisionArray b) {
        for (int i = 0; i < re.length; i++) {
            final float x1 = re[i];
            final float y1 = im[i];
            final float x2 = b.getRe(i);
            final float y2 = b.getIm(i);

            re[i] = x1 * x2 - y1 * y2;
            im[i] = y1 * x2 + x1 * y2;
        }
    }


    // a = a - b, a=a.minus(b)
    public void minus(ComplexSinglePrecisionArray b) {
        for (int i = 0; i < re.length; i++) {
            re[i] -= b.getRe(i);
        }
        for (int i = 0; i < re.length; i++) {
            im[i] -= b.getIm(i);
        }
    }

    public float[] logScaleMagnitude(float magScale) {
        int N = re.length;
        float result[] = new float[N];
        for (int i = 0; i < N; i++) {
            result[i] = (float) (magScale *
                    Math.log(1 + Math.sqrt(re[i] * re[i] + im[i] * im[i])));
            if (result[i] > 255)
                result[i] = 255;
        }
        return result;
    } // End of function magnitude().

    public float[] getIm() {
        return im;
    }

    public float getIm(int i) {
        return im[i];
    }

    public void setIm(float[] im) {
        this.im = im;
    }

    public float getRe(int i) {
        return re[i];
    }

    public float[] getRe() {
        return re;
    }

    public int getLength() {
        return re.length;
    }

    public void setRe(float[] re) {
        this.re = re;
    }
}
